package com.oqpo.api.web.dto.response.stinfo.item;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.oqpo.api.entity.stinfo.ItemEntity;
import com.oqpo.api.entity.stinfo.StoreItemUnitEntity;
import com.oqpo.api.web.dto.request.stinfo.item.SaveItemListData;
import com.oqpo.api.web.dto.response.stinfo.itemUnit.StoreItemUnitInfoResponse;
import com.oqpo.api.web.dto.response.stinfo.spply.SpplyInfoResponse;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;

import javax.validation.constraints.NotBlank;
import java.util.List;
import java.util.stream.Collectors;

@Getter
@Setter
@Builder
@AllArgsConstructor(access = AccessLevel.PROTECTED)
@JsonInclude(JsonInclude.Include.ALWAYS)
public class ChkUploadItemListResponse {

    @ApiModelProperty(value = "성공건수")
    private int successCnt;

    @ApiModelProperty(value = "실패건수")
    private int errorCnt;

    @ApiModelProperty(value = "품목 리스트")
    private List<ChkItemListData> itemList;


    public static ChkUploadItemListResponse toDTO(int success, int error, List<SaveItemListData> entities) {
        if (entities == null) return null;

        return ChkUploadItemListResponse.builder()
                .successCnt(success)
                .errorCnt(error)
                .itemList(entities.stream()
                        .map(m -> ChkItemListData.builder()
                                .viewCd("C")
                                .resultCd(m.getResultCd())
                                .resultMsg(m.getResultMsg())
                                .itemNm(m.getItemNm())
                                .weight(m.getWeight())
                                .weightUnitNm(m.getWeightUnitNm())
                                .volume(m.getVolume())
                                .volumeUnitNm(m.getVolumeUnitNm())
                                .width(m.getWidth())
                                .widthUnitNm(m.getWeightUnitNm())
                                .height(m.getHeight())
                                .heightUnitNm(m.getHeightUnitNm())
                                .high(m.getHigh())
                                .highUnitNm(m.getHighUnitNm())
                                .expryYn(m.getExpryYn())
                                .expryPeriod(m.getExpryPeriod())
                                .rtnDvsnNm(m.getRtnDvsnNm())
                                .impDvsnNm(m.getImpDvsnNm())
                                .podrPssblDvsnNm(m.getPodrPssblDvsnNm())
                                .pchItemYn(m.getPchItemYn())
                                .saleItemYn(m.getSaleItemYn())
                                .itemTypeNm(m.getItemTypeNm())
                                .unitNm(m.getUnitNm())
                                .rtnPssblYn(m.getRtnPssblYn())
                                .barcode(m.getBarcode())
                                .storeDeliTypeNm(m.getStoreDeliTypeNm())
                                .build())
                        .collect(Collectors.toList()))
                .build();
    }

    @Getter
    @Builder
    @AllArgsConstructor(access = AccessLevel.PROTECTED)
    static class ChkItemListData {

        @ApiModelProperty(value = "CRUD")
        private String viewCd;

        @ApiModelProperty(value = "결과코드")
        private String resultCd;

        @ApiModelProperty(value = "결과명")
        private String resultMsg;

        @ApiModelProperty(value = "품목명")
        private String itemNm;

        @ApiModelProperty(value = "무게")
        private Float weight;

        @ApiModelProperty(value = "무게")
        private String weightUnitNm;

        @ApiModelProperty(value = "부피")
        private Float volume;

        @ApiModelProperty(value = "부피단위")
        private String volumeUnitNm;

        @ApiModelProperty(value = "가로")
        private Float width;

        @ApiModelProperty(value = "가로단위")
        private String widthUnitNm;

        @ApiModelProperty(value = "세로")
        private Float height;

        @ApiModelProperty(value = "세로단위")
        private String heightUnitNm;

        @ApiModelProperty(value = "높이")
        private Float high;

        @ApiModelProperty(value = "높이단위")
        private String highUnitNm;

        @ApiModelProperty(value = "유통기한여부")
        private String expryYn;

        @ApiModelProperty(value = "유통기한기간")
        private Integer expryPeriod;

        @ApiModelProperty(value = "반품구분코드")
        private String rtnDvsnNm;

        @ApiModelProperty(value = "수입구분코드")
        private String impDvsnNm;

        @ApiModelProperty(value = "발주가능구분")
        private String podrPssblDvsnNm;

        @ApiModelProperty(value = "구매품목여부")
        private String pchItemYn;

        @ApiModelProperty(value = "판매품목여부")
        private String saleItemYn;

        @ApiModelProperty(value = "품목유형")
        private String itemTypeNm;

        @ApiModelProperty(value = "단위")
        private String unitNm;

        @ApiModelProperty(value = "반품가능여부")
        private String rtnPssblYn;

        @ApiModelProperty(value = "바코드")
        private String barcode;
        @ApiModelProperty(value = "매장배송구분")
        private String storeDeliTypeNm;

    }
}
